function X = datagen(Loi,a,b,Ndata,Nens)
%
% DATAGEN :          Gnration d'ensembles de donnes de 
%    taille Ndata selon une loi normale d'esprance a et de
%    variance b^2 : N(a,b^2) ou selon une loi uniforme dans
%    l'intervalle [a b] : U(a,b)
%
% X = datagen(Loi,a,b,Ndata,Nens);
%
% Entres :
% ---------
% Loi   : Loi de probabilit. 2 cas possibles :
%           'norm' : loi normale
%           'unif' : loi uniforme
% a, b  : paramtres de la loi : 
%           pour la loi normale  : Esprance et cart type
%           pour la loi uniforme : borne inf et sup de l'intervalle
% Ndata : Nombre de donnes
% Nens  : Nombre d'ensembles  constituer si ce dernier parametre
%         n'est pas pass, sa valeur par dfaut est 1 (un seul 
%         ensemble de Ndata gnr).
% 
% Sorties :
% ---------
% X : Matrice de dimension Ndata par Nens valorise selon
%     la loi demande.

%_________________________________________________________
% Auteur : C.Sorror (fevrier 2012)
% --------------------------------------------------------
if (nargin<5) Nens=1; end   % 1 ensemble par dfaut
%
if (Loi=='norm')
    X = randn(Ndata,Nens);  % N(moyenne=0, variance=1)
    X = (b*X) + a;          % N(moyenne=a, variance=b^2)
elseif (Loi=='unif')
    X  = rand(Ndata,Nens);  % uniforme sur [0 1]
    %X = X*b;               % uniforme sur [0 b]
    %X = X*(b-a);           % uniforme sur [0 (b-a)]
    X  = a + X*(b-a);       % uniforme sur [0+a  (b-a)+a] => sur [a b]
else
   error('datagen: la Loi doit etre ''norm'' ou ''unif''');
end    
%
% --------------------------------------------------------
%
