%--------------------------------------------------------------
% Script de correction pour le TP de statistiques descriptives 
% sur des donnes unidimentionnelles (voir sommaire ci-dessous)

% _____________________________________________________________	
% Author : C. Sorror (octobre 2012)
%--------------------------------------------------------------
clear all; close all;  clc
%
%=============================================================
%          S O M M A I R E 
%          ----------------
% 1) n Lancs d'un d
% 2) Histogramme normalis et fonction de densit
% 2.1) Utilisation de donnes unidimensionnelles simules
% 2.2) Illustration du thorme de la limite centrale (TCL)
% 2.3) Convergence de la densit empirique vers la thorique 
% 3) Donnes relles de pluie au Sngal
% 3.1) Indicateurs statistiques lmentaires
% 3.2) Encadrement  un cart type
% 3.3) Quartiles
choix = 0;
%
fprintf('\n'); 
%=============================================================
if (choix==1 | choix==0)
disp('1) Lancs d''un d n fois');
%=============================================================
% Les nombres n de lancs du d
NbLances=[10,  20, 50, 100,  500,  2000];
NbExp   =  length(NbLances); % Nombre d'expriences
% 
rand('seed',0);  % (pour reproduire  chaque fois les memes rsultats, ce qui n'est pas une obligation)
h=figure; set(h,'color','w');
for i=1:NbExp
    %
    subplot(2,ceil(NbExp/2),i);
    %
    % Lanc du d n fois (n=NbLances(i))
    X = ceil(rand(NbLances(i),1)*6);
    Xmoy(i) = mean(X);      % Les moyennes empiriques
    N = hist(X,NbExp);      % La rpartition dans chaque classe
    %
    % Calculs {Frquences, Pourcentages, Effectifs} par parts
    % et des indicateurs sur ces rsultats
    Pcent   = round((100*N)/sum(N)); % Les poucentages (arrondis)
    Pmin(i) = min(Pcent);            % Les min 
    Pmax(i) = max(Pcent);            % et les max
    Pec(i)  = std(Pcent);            % Les carts-types

    % Mise en forme des labels des parts 
    for k=1:NbExp
        if (exist('OCTAVE_VERSION'))
           pielab{k} = sprintf('%d %.0f%c',k,Pcent(k),'%'); % Label sur une ligne ... ou sur
        else 
           pielab{k} = [ {sprintf('%d',k)}, {sprintf('%.0f%c',Pcent(k),'%')} ]; % 2 colonnes
        end
    end
  
    % The calendos
    h = pie(N,pielab); % itself
    szfont=12; %11 12 16 18
    textObjs = findobj(h,'Type','text');
    set(textObjs,'fontsize',szfont, 'fontweight','demi');
    title(sprintf('%d lancs',NbLances(i)),'fontsize',szfont,'fontweight','demi');
end
%
disp('Moyennes des lancs de ds : '); Xmoy
%
% Courbes des indicateurs en fonction du nombre de lancs
figure; hold on
plot([1:NbExp],[Pmax; Pmin; Pmax-Pmin; Pec],'-o');
%
legend('%max','%min','%gap','%ec');
%
%modification des labels sur l'axes des x courants :
set(gca,'Xtick',[1:NbExp],'XTickLabel',NbLances)
%
xlabel('nombres de lancs');
title('Evolution des indicateurs sur les pourcentages');
%
end % choix 1
%
%=============================================================
if (choix==21 | choix==2 | choix==0)
disp('2.1) Utilisation de donnes unidimensionnelles simules')
%-------------------------------------------------------------
Ndata = 500;     % Taille des donnes
%
% CAS d'une loi normale N(mu,sig2) -----------------
mu   = 3;              % Esprance  Thorique
sig2 = 5;              % Variance   Thorique
sig  = sqrt(sig2);     % Ecart type Thorique
randn('seed',0);       % Init du random (si on veut)
X = datagen('norm',mu,sig,Ndata);  % Gnration des donnes
fprintf('Donnes gaussiennes : moyenne=%f, variance=%f\n',mean(X),var(X));
%
figure; hist(X);
title(sprintf('Histogramme : %d donnes ~> N(%d,%d)',Ndata,mu,sig2));
%
figure; hncfd('norm',mu,sig,X);  
title(sprintf('Histogramme normalis : %d donnes ~> N(%d,%d)',Ndata,mu,sig2));
%
% CAS d'une loi uniforme U(a,b) ----------------------
a   = 3;               % Borne inf
b   = 5;               % Borne sup
rand('seed',0);        % Init du random (si on veut)
X = datagen('unif',a,b,Ndata); % Gnration des donnes
fprintf('Donnes uniformes : minimum=%f, maximum=%f\n',min(X),max(X));
%
figure; hist(X);
title(sprintf('Histogramme : %d donnes ~> U(%d,%d)',Ndata,a,b));
%
figure; hncfd('unif',a,b,X);  
title(sprintf('Histogramme normalis : %d donnes ~> U(%d,%d)',Ndata,a,b));
%
end % choix 21
%
%-------------------------------------------------------------
if (choix==22 | choix==2 | choix==0)
disp('2.2) Illustration du thorme de la limite centrale (TCL)')
%-------------------------------------------------------------
% Cas de donnes gnres selon la Loi Uniforme
a    = 3;              % Borne inf
b    = 5;              % Borne sup
mu   = (a+b)/2;        % Esprance Thorique    
sig2 = ((b-a)^2)/12;   % Variance  Thorique    
sig  = sqrt(sig2);     % Ecart Type Thorique   
%
Ntir = [10, 200];         % Nombre de tirages
Ndataset = [5, 50, 500];  % Nombre de donnes par tirage
rand('seed',0);           % Init du random (si on veut)
%
%-------------------------------------------------------------
% - Histogrammes et fonctions de densit pour le TCL
%-------------------------------------------------------------
figure;
for i=1:length(Ndataset)
    TCLsig = sig/sqrt(Ndataset(i));  % Ecart type de la loi normale des moyennes    
    %
    for j=1:length(Ntir)
        subplot(3,2,j+length(Ntir)*(i-1));      % avec subplot, on y arrive,... 
        %subplot(6,1,i+length(Ndataset)*(j-1)); % ...de diffrente manire, ... 
        hold on;                                % ...mais c'est pas simple. 

        X = datagen('unif',a,b,Ndataset(i),Ntir(j));  % Generation des donnes
        Xbar = mean(X);                  % Moyenne des tirages
        hncfd('norm',mu,TCLsig,Xbar,[0.9 0.9 0.75]);  % Histogrammes des moyennes
        axis([3.2 4.8 ylim]);                         % To adjust axis
        title(sprintf('%d tirages de %d donnes',Ntir(j),Ndataset(i)));
        
        Tmoy(i,j) = mean(Xbar); % Stockage des moyennes et des  ...
        Tvar(i,j) = var(Xbar);  % ... variabnces empiriques de Xbar
    end
    Tvtheo(i) = TCLsig^2;       % Stockage des variances thoriques
end
% placer un titre sur la figure
text(0.8, 78, sprintf('TCL : Histogrammes normaliss des moyennes des tirages selon une Loi U(%d,%d)',a,b));
%
%-------------------------------------------------------------
% - Figures complmentaires :
%-------------------------------------------------------------
% Affichage des moyennes pour chaque tirage p et en fonction
% du nombre de donnes n (empirique et thorique)
figure;
subplot(1,2,1); hold on; 
plot([mu*ones(3,1), Tmoy],'*-','linewidth',1.5);
set(gca,'XTick',[1:length(Ndataset)],'XTickLabel',Ndataset);
legend('Thorique','Empirique p=10','Empirique p=200');
xlabel('Nombre de donnes (n)');
title('Moyenne selon le nombre de donnes par tirage (p)');
%
% Affichage des variance pour chaque tirage p et en fonction
% du nombre de donnes n (empirique et thorique)
subplot(1,2,2); hold on; 
plot([Tvtheo', Tvar],'*-','linewidth',1.5);
set(gca,'XTick',[1:length(Ndataset)],'XTickLabel',Ndataset);
legend('Thorique','Empirique p=10','Empirique p=200');
xlabel('Nombre de donnes (n)');
title('Variance selon le nombre de donnes par tirage (p)');
%
end % choix 22
%
%-------------------------------------------------------------
if (choix==23 | choix==2 | choix==0)
disp('2.3) Convergence de la densit empirique vers la thorique')
%-------------------------------------------------------------
% On recommence le meme type d'exprience pour voir la densit 
% empirique se rapprocher progressivement de la thorique.
% (Cette fois on ne fait pas apparaitre d'histogramme)
a    = 3;              % Borne inf
b    = 5;              % Borne sup
mu   = (a+b)/2;        % Esprance Thorique    
sig2 = ((b-a)^2)/12;   % Variance  Thorique    
sig  = sqrt(sig2);     % Ecart Type Thorique  
%
Ndataset   = [10];     % Nombre de donnes par tirage
Nbdatasize = length(Ndataset); 
Ntir  = [5, 10, 20, 30, 40, 50, 100, 200, 500, 1000, 2000, 5000]; 
NbExp = length(Ntir)*length(Ndataset);
rand('seed',0);         % Init du random (si on veut)
%
figure;
for i=1:Nbdatasize 
  TCLsig = sig/sqrt(Ndataset(i));  % Ecart type de la loi normale des moyennes    
  for j=1:length(Ntir) 
    subplot(Nbdatasize*3,ceil(NbExp/Nbdatasize/3),j+length(Ntir)*(i-1));  hold on;   
    X = datagen('unif',a,b,Ndataset(i),Ntir(j));  % Generation des donnes
    Xbar = mean(X);                  % Moyenne des tirages
    hncfd('norm',mu,TCLsig,Xbar,0);  % Courbes des densits thorique et empirique
    %
    axis([3.5 4.5 0.0 normpdf(mu,mu,TCLsig)+0.5]); set(gca,'fontweight','bold');
    title(sprintf('%d tirages de %d donnes',Ntir(j),Ndataset(i)),'fontweight','bold');
  end
end
% placer un titre sur la figure
text(0.0, 6.6, sprintf('Convergence de la densit empirique vers la densit thorique',a,b));
%
end % choix 23
%
fprintf('\n'); 
%=============================================================
if (choix==31 | choix==32 | choix==33 | choix==3 | choix==0)
disp('3) Donnes relles de cumul de pluie au Sngal');
%-------------------------------------------------------------
% Chargement des donnes et initialisation
Data   = load('pluie.txt');
Annees = Data(:,1);        % Annes
Data   = Data(:,2:end);    % Donnes (brute) des prcipitations
[n,m]  = size(Data);       % Dim
%
% Calculs statistiques lmentaires
moyx = mean(Data);         % Moyenne
stdx = std(Data);          % Ecart type
end
%-------------------------------------------------------------
if (choix==31 | choix==3 | choix==0)
disp('3.1) Indicateurs statistiques lmentaires :');
%-------------------------------------------------------------
varnames = {'zone1','zone2','zone3','zone4'}; % noms des variables
%
% Calculs statistiques lmentaires (suite)
minx = min(Data);       % Min
maxx = max(Data);       % Max
etdx = maxx-minx;       % Etendu
format short g
fprintf('           Zone     moyenne    cart type      min          max        etendue \n');
disp([[1:4]', moyx', stdx', minx', maxx', etdx']);
%
% Visualisation des donnes brutes :
% - Graphe de chaque variable : 
hf=figure; set(hf, 'Color','w'); hold on 
hp=plot(Annees,Data,'-x'); set(hp,'LineWidth',1.5);
% - Habillage de la figure : 
legend(varnames);
title('Cumuls de precipitations annuelles au Sngal (mm)')
axis tight; box on, grid on
%
% Prcipitation moyennes (non demand)
[n,m]  = size(Data);           % Dim
PrecipMoy = (sum(Data')/m)';   % ou: PrecipMoy = (sum(Data,2)/m);
hp=plot(Annees,PrecipMoy,'k');
varnames(5) = {'moy'};
legend(varnames);
%
end % choix 31
%
%-------------------------------------------------------------
if (choix==32 | choix==3 | choix==0)
disp('3.2) Encadrement  un cart type');
%-------------------------------------------------------------
% Pour chaque zone, faire une figure avec les prcipitations 
% et leur encadrement par un cart type autour de la moyenne
% + indiquer le pourcentage des points inclus dans l'encadrement
for i=1:4;
  figure; hold on;
  plot(Annees,Data(:,i),'b-*'); % Les donnes
  %
  % Ligne de la moyenne et des encadrement  +/- 1 cart type
  line([Annees(1) Annees(end)],[moyx(i), moyx(i)], 'color','k')
  line([Annees(1) Annees(end)],[moyx(i)+stdx(i), moyx(i)+stdx(i)], 'color','r');
  line([Annees(1) Annees(end)],[moyx(i)-stdx(i), moyx(i)-stdx(i)], 'color','r');
  %
  % Pourcentage de points inclus dans l'encadrement :
    %In  = find( Data(:,i)<(moyx(i)+stdx(i)) & Data(:,i)>(moyx(i)-stdx(i)) ); % OU
  In   = find(abs(Data(:,i)-moyx(i))<stdx(i)); % Le nombre de points
  pcin = (length(In)/n)*100;                   % Le pourcentage
  title(sprintf('zone%.1i : encadrement par un cart type : %.2f%c de points inclus', i,pcin,'%'))
  axis tight;
  %
end
%
end % choix 32
%
%-------------------------------------------------------------
if (choix==33 | choix==3 | choix==0)
disp('3.3) Quartiles :');
%-------------------------------------------------------------
Ptils = [25, 50, 75];           % Les quantiles qui nous intressent (Q1,Q2,Q3)
Qtils = prctile(Data,Ptils);    % Calcul de ces quantiles
%
% Affichage des rsultats
disp('             zone1   zone2   zone3   zone4');
for i=1: length(Ptils)
    fprintf('  Q%i(%.0f)   : %.2f  %.2f  %.2f  %.2f \n', i,Ptils(i),Qtils(i,:)');
end
%
% Boites  moustaches
moustache(Data);
xlabel('Zones','fontweight','demi','fontsize',14);
ylabel('Valeurs','fontweight','demi','fontsize',14);
title('Boites  moustaches','fontweight','demi','fontsize',14);
set(gca,'fontweight','bold','fontsize',12);
set(gcf,'color','w');
%
% Ecart interquatile
interQecart = Qtils(3,:) - Qtils(1,:); 
fprintf('Q(3)-(Q(1) : %.2f  %.2f  %.2f  %.2f \n', interQecart );
%
end % choix 33
%
%=============================================================
%



